from pathlib import Path
from azureml.pipeline.steps import PythonScriptStep
from azureml.core.runconfig import RunConfiguration

def save_step(
    compute_target,
    environment,
    img_dir,
    save_datastore_name,
    path_on_datastore,
    chunk_index,
    name="Save"
):
    """
    This step will save images to a DataStore in a specified Dataset,
    creating it if it doesn't exist.

    Args:
        workspace: The current workspace
        compute_target: The compute target to run the step on
        environment: The environment in which to run the script
        img_dir: Reference to the directory containing the raw images
        target_dataset_name: The name of the target dataset
        save_datastore_name: The name of the Datastore to save data on
        path_on_datastore: Path on the datastore
        name: Action name of the node

    Returns:
        The step dictionary
    """

    # Create the run configuration
    run_config = RunConfiguration()
    # Assign the cluster to the run configuration
    run_config.target = compute_target
    # Assign the environment to the run configuration
    run_config.environment = environment
    print("Run configuration created for the images saving step")

    # Step 2, run the data saving script
    step = PythonScriptStep(
        script_name="save.py",
        name=name,
        arguments=[
            "--img_dir",
            img_dir,
            "--chunk_index",
            chunk_index,
            "--save_datastore_name",
            save_datastore_name,
            "--save_datastore_path",
            path_on_datastore,

        ],
        inputs=[img_dir],
        compute_target=compute_target,
        runconfig=run_config,
        source_directory=Path(__file__).resolve().parent,
        allow_reuse=True,
    )

    return step



import argparse

from azureml.core import Dataset, Run
from chunk_files import chunk_files
import shutil

# Get parameters
parser = argparse.ArgumentParser()
parser.add_argument("--dataset_name", type=str, help="Dataset name")
parser.add_argument("--dataset_version", type=str, help="Dataset version")
parser.add_argument(
    "--raw_data_dir", type=str, help="Directory to store raw pdf,png,jpeg,tiff"
)
parser.add_argument("--number_chunk", type=int, help="Number of chunk")
parser.add_argument("--chunk_index", type=int, help="Chunk index")

# Get arguments from parser
args = parser.parse_args()
dataset_name = args.dataset_name
dataset_version = args.dataset_version
raw_data_dir = args.raw_data_dir
number_chunk = args.number_chunk
chunk_index = args.chunk_index

# Get the experiment run context
run = Run.get_context()
workspace = run.experiment.workspace

# Get raw data
raw_dataset = Dataset.get_by_name(workspace, dataset_name, version=dataset_version)
print(
    f"Retrieve raw data in dataset: {raw_dataset.name} - version: {raw_dataset.version}"
)

raw_dataset.as_mount("/mnt/ged_cni")

chunk_files(raw_dataset, raw_data_dir, number_chunk, shutil.copy, [chunk_index])





import subprocess


def run_ml_cli(files_directory: str,
               output_directory: str,
               ml_cli_template: str,
               process_path: str = "/home/ml-cli/Ml.Cli",
               run_path: str = "/home/ml-cli",
               tasks_path: str = "/home/tasks.json"):
    ml_cli_template = ml_cli_template.replace("${files_directory}$", files_directory)
    ml_cli_template = ml_cli_template.replace("${output_directory}$", output_directory)
    print("ml_cli_template")
    print(ml_cli_template)

    with open(tasks_path, 'w') as file:
        file.write(ml_cli_template)

    # r'C:\github\ml-cli\src\Ml.Cli\bin\Debug\net6.0\Ml.Cli.exe'
    args = [process_path, "--base-path", "/", '--tasks-path', tasks_path]
    with subprocess.Popen(args,
                          cwd=run_path,
                          stdout=subprocess.PIPE,
                          stderr=subprocess.STDOUT) as process:
        for line in process.stdout:
            print(line.decode('utf8'))



from pathlib import Path
from azureml.pipeline.steps import PythonScriptStep
from azureml.core.runconfig import RunConfiguration
from azureml.pipeline.core import PipelineData


def data_ingestion_step(
    compute_target,
    environment,
    datastore,
    dataset_name,
    dataset_version,
    number_chunk:int,
    chunk_index:int,
    name="Ingest Data",
):
    """
    This step will ingest data from the specified dataset and pass it to
    the next step as a temporary data reference.

    Args:
        compute_target: The compute target to run the step on
        environment: The environment in which to run the script
        datastore: The datastore that will be used
        name: Action name of the node

    Returns:
        The step, the step outputs dictionary
    """

    run_config = RunConfiguration()
    run_config.target = compute_target
    run_config.environment = environment
    print("Run configuration created for the data ingestion step")

    raw_data_dir = PipelineData(
        name="raw_data_dir",
        datastore=datastore,
        is_directory=True,
    )

    outputs = [raw_data_dir]
    outputs_map = {"raw_data_dir": raw_data_dir}

    step = PythonScriptStep(
        script_name="data_ingestion.py",
        name=name,
        arguments=[
            "--dataset_name",
            dataset_name,
            "--dataset_version",
            dataset_version,
            "--raw_data_dir",
            raw_data_dir,
            "--number_chunk",
            number_chunk,
            "--chunk_index",
            chunk_index
        ],
        outputs=outputs,
        compute_target=compute_target,
        source_directory=Path(__file__).resolve().parent,
        runconfig=run_config,
        allow_reuse=True,
    )

    return step, outputs_map



import argparse
import json
from pathlib import Path
from azureml.core import Run, Dataset

from aggregate_output import create_summary
from azureml.core.datastore import Datastore
from azureml.data.datapath import DataPath

# Get parameters
parser = argparse.ArgumentParser()
parser.add_argument(
    '--img_dir', type=str, help='Directory where images to save are stored'
)
parser.add_argument('--save_datastore_name', type=str, help='save datastore name')
parser.add_argument('--save_datastore_path', type=str, help='save datastore path')

parser.add_argument('--save_dataset', type=str, help='save dataset')
parser.add_argument('--chunk_index', type=str, help='index of the current chunk')

# Get arguments from parser
args = parser.parse_args()
img_dir = args.img_dir
# datastore_name = args.datastore_name
chunk_index = args.chunk_index
save_datastore_name = args.save_datastore_name
save_datastore_path = args.save_datastore_path

# Get the experiment run context
run = Run.get_context()
workspace = run.experiment.workspace

# Load data
print("Loading Data...")
data_output_dir_path = Path(img_dir)

summary_filename = "summary_chunk" + str(chunk_index) + ".json"
chunk_name = "chunk" + str(chunk_index)
json_chunk_name = "json" + chunk_name
output_jsons = str(data_output_dir_path / json_chunk_name)
create_summary(output_jsons, str(data_output_dir_path / summary_filename))

dstore = Datastore.get(workspace, save_datastore_name)
dataset = Dataset.File.upload_directory(
    src_dir=str(data_output_dir_path), target=DataPath(dstore, save_datastore_path), pattern="*", show_progress=True,
    overwrite=True
)

dir_list = [p for p in Path(output_jsons).iterdir() if p.is_dir()]

for dir_path in dir_list:
    dir_name = dir_path.name
    source = str(data_output_dir_path / dir_name)
    dummy_path = str(data_output_dir_path / dir_name / 'dummy.json')
    with open(dummy_path, 'w') as f:
        json.dump({}, f, indent=2)

    Dataset.File.upload_directory(
        src_dir=source, target=DataPath(dstore, str(Path(save_datastore_path) / dir_name)), pattern="*",
        show_progress=True,
        overwrite=True
    )

    subdir_list = [p for p in Path(source).iterdir() if p.is_dir()]

    for subdir_path in subdir_list:
        subdirname = subdir_path.name
        destination = str(Path(save_datastore_path) / dir_name / subdirname)
        sub_source = str(Path(subdir_path) / dir_name / subdirname)
        Dataset.File.upload_directory(
            src_dir=sub_source, target=DataPath(dstore, destination), pattern="*",
            show_progress=True,
            overwrite=True
        )

# End the run
run.complete()



from datetime import datetime

from azureml.pipeline.core import Pipeline

from ..modules.ingestion_chunk.data_ingestion_step import data_ingestion_step
from ..modules.extraction.extraction_step import extraction_step
from ..modules.save.save_step import save_step


def train_pipeline(workspace, datastore, clusters, environments, dataset_dict, mlcli_template:str):

    number_chunk = dataset_dict["inputs"]["number_chunk"]
    start_at_chunk_index = dataset_dict["inputs"]["start_at_chunk_index"]

    steps = []
    previous_data_extraction_step = None
    previous_previous_data_extraction_step = None
    previous_data_injection_step = None
    for chunk_index in range(number_chunk):
        if start_at_chunk_index > chunk_index:
            continue
        # Data ingestion
        data_ingestion, data_ingestion_outputs = data_ingestion_step(
            clusters[0],
            environments[0],
            datastore,
            dataset_dict["inputs"]["dataset_name"],
            dataset_dict["inputs"]["dataset_version"],
            name="Data Ingestion",
            number_chunk=number_chunk,
            chunk_index=chunk_index
        )
        steps.append(data_ingestion)

        if previous_data_injection_step is not None:
            data_ingestion.run_after(previous_data_injection_step)

        if previous_previous_data_extraction_step is not None:
            data_ingestion.run_after(previous_previous_data_extraction_step)
        previous_data_injection_step = data_ingestion

        # Data extraction
        data_extraction, data_extraction_outputs = extraction_step(
            clusters[0],
            environments[1],
            datastore,
            data_ingestion_outputs["raw_data_dir"],
            chunk_index,
            mlcli_template,
            name="Data Extraction"
        )
        steps.append(data_extraction)

        if previous_data_extraction_step is not None:
            data_extraction.run_after(previous_data_extraction_step)
        previous_previous_data_extraction_step = previous_data_extraction_step
        previous_data_extraction_step = data_extraction


        path_on_datastore = dataset_dict["outputs"]["path_on_datastore"]
        date_folder_name = datetime.now().strftime("%d-%m-%Y_%Hh%M")
        updated_path_on_datastore = path_on_datastore.replace("{date}", date_folder_name)
        save = save_step(
            clusters[0],
            environments[0],
            data_extraction_outputs["data_output_dir"],
            dataset_dict["outputs"]["datastore_name"],
            updated_path_on_datastore,
            chunk_index,
            name="Save",
        )

        steps.append(save)
        save.run_after(data_extraction)

    pipeline = Pipeline(
        workspace=workspace,
        steps=steps,
    )

    print("Pipeline is built.")
    return pipeline



import setuptools
import version

with open('requirements.txt') as f:
    requirements = f.read().splitlines()

setuptools.setup(
    name='ml-cli-azureml-pipeline',
    version=version.VERSION,
    packages=setuptools.find_packages(exclude=["tests", "tests.*"], where='src'),
    package_dir={'': 'src'},
    install_requires=requirements,
    author="Axa_france",
    author_email="guillaume.chervet@axa.fr",
    url='https://github.com/AxaGuilDEv/ml-cli-azureml-pipeline',
    description="ML-Cli run in an azureML Pipeline",
    long_description="Run our ML integration test automaticaly on production environment so we can compute : prediction quality, response time, infrastructure cost",
    platforms='POSIX',
    classifiers=["Programming Language :: Python :: 3 :: Only",
                 "Programming Language :: Python :: 3.8",
                 "Topic :: Scientific/Engineering :: Information Analysis",
                 ]
)
